// Adjustable universal joints
// Details on:
// https://homofaciens.de/technics-machine-elements-adjustable-joints_en.htm

$fn=100;

M3_hole = 3.4;
M3_nut_e = 6.4;
M4_hole = 4.4;
M4_nut_e = 7.66+0.5;
M5_nut_e = 8.79+0.5;
M6_nut_e = 11.05+0.5;

// Ball bearing dimensions
bb1_do = 19;
bb1_di = 6;
bb1_h = 6;

bb2_do = 30;
bb2_di = 10;
bb2_h = 9;



  joint_wide(1);
  translate([0, 50, 0]){
    joint_wide(0);
  }

  translate([0, -50, 0]){
    joint_small(1);
  }
  translate([0, -90, 0]){
    joint_small(0);
  }

  translate([45, 0, 0]){
    bearing_mount();
  }
  translate([45, 45, 0]){
    bearing_mount(100);
  }
  translate([45, -55, 0]){
    hand_crank();
  }

  translate([-65, -65, 0]){
    joint_ball();
  }

module hand_crank(){
  difference(){
    cylinder(d=60, h=2, center=true);
    cylinder(d=5.2, h=2.1, center=true);
    translate([25, 0, 0]){
      cylinder(d=M3_hole, h=2.1, center=true);
    }
    for(i=[0:5:360]){
      rotate([0, 0, i]){
        translate([0, 30, 0]){
          cylinder(d=2, h=2.1, center=true);
        }
      }
    }
  }
}

module bearing_mount(bm_h = 40){
  bb_di = 5;
  bb_do = 16;
  bb_h = 5;
  
  bm_w = 27;
  bm_thick = 12;
  bm_wall = 2;
  bm_foot = 40;
  
  mount_d = (bm_foot-10)/2;
  
  difference(){
    union(){
      roundedRect([bm_foot, bm_foot, bm_wall], 5);
      translate([0, 0, bm_h/2]){
        cube(size=[bm_thick, bm_w, bm_h], center=true);
      }
      translate([0, 0, bm_h]){
        rotate([0, 90, 0]){
          cylinder(d=bm_w, h=bm_thick, center=true);
        }
      }
    }
    translate([0, 0, bm_h]){
      rotate([0, 90, 0]){
        cylinder(d=bb_do, h=bm_thick*1.1, center=true);
      }
    }
    translate([mount_d, mount_d, bm_wall/2]){
      cylinder(d=M3_hole, h=bm_wall*1.1, center=true);
    }
    translate([-mount_d, mount_d, bm_wall/2]){
      cylinder(d=M3_hole, h=bm_wall*1.1, center=true);
    }
    translate([-mount_d, -mount_d, bm_wall/2]){
      cylinder(d=M3_hole, h=bm_wall*1.1, center=true);
    }
    translate([mount_d, -mount_d, bm_wall/2]){
      cylinder(d=M3_hole, h=bm_wall*1.1, center=true);
    }
  }
  
}

module joint_ball(ball=1){

  // Push rods
  pr_d = 6;

  // Hinge dimensions
  j_ball_d = 25;
  j_ball_cut = 3;
  j_cube = 15;
  j_head_d = 10;
  j_gap_w = j_ball_d - 5;
  j_gap_l = 20;
  j_h = 20;
  j_l = 50;
  j_w = 40;
  j_axis = 8;
  j_screw = 25;

  if(ball){
    // Center ball
    translate([0, 40, j_ball_d/2-j_ball_cut]){
      difference(){
        sphere(d=j_ball_d, center=true);
        translate([0, 0, (j_ball_d-j_ball_cut)/2]){
          cube(size=[j_ball_d, j_ball_d, j_ball_cut], center=true);
        }
        translate([0, 0, -(j_ball_d-j_ball_cut)/2]){
          cube(size=[j_ball_d, j_ball_d, j_ball_cut], center=true);
        }
        cylinder(d=M4_hole, h=j_ball_d, center=true);
        translate([0, 0, j_ball_d/2-j_ball_cut]){
          cylinder(d=M4_nut_e, h=5, center=true, $fn=6);
        }
      }
    }
  }

  // fork
  translate([-15, 0, j_h/2]){
    difference(){
      translate([0, 0, -j_h/2]){
        roundedRect([j_l, j_w, j_h], 15, [1,0,1,0]);
      }
      // Center gap cut
      translate([(j_l-j_gap_l)/2+0.1, 0, 0]){
        cube(size=[j_gap_l, j_gap_w, j_h*1.1], center=true);
      }
      translate([j_l/2-j_gap_l, 0, 0]){
        scale([1.7,1,1]){
          cylinder(d=j_gap_w, h=j_h*1.1, center=true);
        }
      }
      // Push rod cut
      translate([-j_l/2, 0, 0]){
        rotate([0, 90, 0]){
          cylinder(d=pr_d, h=2*(j_l-j_gap_l)-2, center=true);
        }
      }
      // Ball cut
      translate([j_l/2-j_axis, 0, 0]){
        sphere(d=j_ball_d, center=true);
      }
      translate([j_l/2-j_axis, 0, 0]){
        rotate([90, 0, 0]){
          cylinder(d=12, h=j_w*1.1, center=true);
        }
      }
      // Adjustment screw cut
      translate([j_l/2-j_screw, 0, 0]){
        rotate([90, 0, 0]){
          cylinder(d=M4_hole+1, h=j_w*1.1, center=true);
          translate([0, 0, j_w/2]){
            cylinder(d=M4_nut_e, h=6, center=true, $fn=6);
          }
        }
      }
      // Bend hinge cut
      translate([j_l/2-50, 20, 0]){
        scale([1.3, 1, 2]){
          cylinder(d=30, h=j_h*1.1, center=true);
        }
      }
    }

    // Push rod mount
    prm_wall = 2;
    prm_l = 20;
    prm_longhole = 10;
    translate([-(j_l+prm_l)/2, 0, 0]){
      difference(){
        cube(size=[prm_l, pr_d+2*prm_wall, j_h], center=true);
        cube(size=[prm_l*1.1, pr_d, j_h*1.1], center=true);
        hull(){
          translate([prm_longhole/2, 0, 0]){
            rotate([90, 0, 0]){
              cylinder(d=M4_hole, h=j_w, center=true);
            }
          }
          translate([-prm_longhole/2, 0, 0]){
            rotate([90, 0, 0]){
              cylinder(d=M4_hole, h=j_w, center=true);
            }
          }
        }
      }
    }
  }

  
}

module joint_small(cube = 1, type = 2){
  // Push rods
  pr_d = 5.3;
  
  // Joint dimensions small
  j_ball_d = 20;
  j_ball_cut = 5;
  j_cube = 17.5;
  j_cube_h = 12;
  j_head_d = 10;
  j_gap_w = 20;
  j_gap_l = 19;
  j_h = 10;
  j_l = 50;
  j_w = 30;
  j_axis = 5;
  j_screw = 25;  
  
  if(cube==1){
    // cube
    translate([0, 0, j_cube_h/2]){
      difference(){
        translate([0, 0, -j_cube_h/2]){
          roundedRect([j_cube, j_cube, j_cube_h], 5);
        }
        rotate([90, 0, 0]){
          cylinder(d=5, h=j_cube*1.1, center=true);
        }
        rotate([0, 90, 0]){
          cylinder(d=5, h=j_cube*1.1, center=true);
        }
        pan_shift = 3;
        translate([0, (j_cube+j_head_d)/2-pan_shift, 0]){
          sphere(d=j_head_d, center=true);
        }
        translate([0, pan_shift-(j_cube+j_head_d)/2, 0]){
          sphere(d=j_head_d, center=true);
        }
        translate([(j_cube+j_head_d)/2-pan_shift, 0, 0]){
          sphere(d=j_head_d, center=true);
        }
        translate([pan_shift-(j_cube+j_head_d)/2, 0, 0]){
          sphere(d=j_head_d, center=true);
        }
      }
    }
  }
  
  // fork
  translate([-19, 0, j_h/2]){
    difference(){
      translate([0, 0, -j_h/2]){
        roundedRect([j_l, j_w, j_h], 15, [1,0,1,0]);
      }
      // Gap cut
      translate([(j_l-j_gap_l)/2+0.1, 0, 0]){
        cube(size=[j_gap_l, j_gap_w, j_h*1.1], center=true);
      }
      translate([j_l/2-j_gap_l, 0, 0]){
        scale([1.7,1,1]){
          cylinder(d=j_gap_w, h=j_h*1.1, center=true);
        }
      }
      // Push rod cut
      translate([-j_l/2, 0, 0]){
        rotate([0, 90, 0]){
          cylinder(d=pr_d, h=2*(j_l-j_gap_l)-2, center=true);
          if(type == 2){
            translate([0, 0, j_l/2])rotate([0, 0, 30]){
              cylinder(d=M5_nut_e, h=26, center=true, $fn=6);
            }
          }
        }
      }
      // Pivot screws cut
      translate([j_l/2-j_axis, 0, 0]){
        rotate([90, 0, 0]){
          cylinder(d=M4_hole, h=j_w*1.1, center=true);
          translate([0, 0, j_w/2]){
            cylinder(d=M4_nut_e, h=6, center=true, $fn=6);
          }
          translate([0, 0, -j_w/2]){
            cylinder(d=M4_nut_e, h=6, center=true, $fn=6);
          }
        }
      }
      // Adjustment screw cut
      translate([j_l/2-j_screw, 0, 0]){
        rotate([90, 0, 0]){
          cylinder(d=M4_hole+1, h=j_w*1.1, center=true);
          translate([0, 0, j_w/2]){
            cylinder(d=M4_nut_e, h=6, center=true, $fn=6);
          }
        }
      }
      // Bend hinge cut
      translate([j_l/2-50, 27.5, 0]){
        scale([1.3, 1, 2]){
          cylinder(d=30, h=j_h*1.1, center=true);
        }
      }
      translate([j_l/2-50, -27.5, 0]){
        scale([1.3, 1, 2]){
          cylinder(d=30, h=j_h*1.1, center=true);
        }
      }
    }

    // Push rod mount
    if(type == 1){// Non threaded rod
      prm_wall = 2;
      prm_l = 20;
      prm_longhole = 10;
      translate([-(j_l+prm_l)/2, 0, 0]){
        difference(){
          cube(size=[prm_l, pr_d+2*prm_wall, j_h], center=true);
          cube(size=[prm_l*1.1, pr_d, j_h*1.1], center=true);
          hull(){
            translate([prm_longhole/2, 0, 0]){
              rotate([90, 0, 0]){
                cylinder(d=M4_hole, h=j_w, center=true);
              }
            }
            translate([-prm_longhole/2, 0, 0]){
              rotate([90, 0, 0]){
                cylinder(d=M4_hole, h=j_w, center=true);
              }
            }
          }
        }
      }
    }
  }

}



module joint_wide(cube = 1, type = 2){
  // Push rods
  pr_d = 5.3;
  
  // Hinge dimensions small
  j_ball_d = 20;
  j_ball_cut = 5;
  j_cube = 17.5;
  j_cube_h = 12;
  j_head_d = 10;
  j_gap_w = 20;
  j_gap_l = 19;
  j_h = 10;
  j_l = 50;
  j_w = 30;
  j_axis = 5;
  j_screw = 25;


  // Hinge dimensions normal
  j_ball_d = 20;
  j_ball_cut = 5;
  j_cube = 27.5;
  j_cube_h = 12;
  j_head_d = 10;
  j_gap_w = 30;
  j_gap_l = 19;
  j_h = 10;
  j_l = 50;
  j_w = 40;
  j_axis = 5;
  j_screw = 25;

  // Joint dimensions wide
  j_ball_d = 20;
  j_ball_cut = 5;
  j_cube = 32.5;
  j_cube_h = 12;
  j_head_d = 10;
  j_gap_w = 35;
  j_gap_l = 19;
  j_h = 10;
  j_l = 50;
  j_w = 45;
  j_axis = 5;
  j_screw = 25;
  
  
  if(cube==1){
    // cube
    translate([0, 0, j_cube_h/2]){
      difference(){
        translate([0, 0, -j_cube_h/2]){
          roundedRect([j_cube, j_cube, j_cube_h], 12);
        }
        rotate([90, 0, 0]){
          cylinder(d=5, h=j_cube*1.1, center=true);
        }
        rotate([0, 90, 0]){
          cylinder(d=5, h=j_cube*1.1, center=true);
        }
        pan_shift = 3;
        translate([0, (j_cube+j_head_d)/2-pan_shift, 0]){
          sphere(d=j_head_d, center=true);
        }
        translate([0, pan_shift-(j_cube+j_head_d)/2, 0]){
          sphere(d=j_head_d, center=true);
        }
        translate([(j_cube+j_head_d)/2-pan_shift, 0, 0]){
          sphere(d=j_head_d, center=true);
        }
        translate([pan_shift-(j_cube+j_head_d)/2, 0, 0]){
          sphere(d=j_head_d, center=true);
        }
      }
    }
  }
  
  // fork
  translate([-19, 0, j_h/2]){
    difference(){
      translate([0, 0, -j_h/2]){
        roundedRect([j_l, j_w, j_h], 15, [1,0,1,0]);
      }
      // Gap cut
      translate([(j_l-j_gap_l)/2+0.1, 0, 0]){
        cube(size=[j_gap_l, j_gap_w, j_h*1.1], center=true);
      }
      translate([j_l/2-j_gap_l, 0, 0]){
        scale([1.0,1,1]){
          cylinder(d=j_gap_w, h=j_h*1.1, center=true);
        }
      }
      // Push rod cut
      translate([-j_l/2, 0, 0]){
        rotate([0, 90, 0]){
          cylinder(d=pr_d, h=2*(j_l-j_gap_l)-2, center=true);
          if(type == 2){
            translate([0, 0, j_l/2])rotate([0, 0, 30]){
              cylinder(d=M5_nut_e, h=26, center=true, $fn=6);
            }
          }
        }
      }
      // Pivot screws cut
      translate([j_l/2-j_axis, 0, 0]){
        rotate([90, 0, 0]){
          cylinder(d=M4_hole, h=j_w*1.1, center=true);
          translate([0, 0, j_w/2]){
            cylinder(d=M4_nut_e, h=6, center=true, $fn=6);
          }
          translate([0, 0, -j_w/2]){
            cylinder(d=M4_nut_e, h=6, center=true, $fn=6);
          }
        }
      }
      // Adjustment screw cut
      translate([j_l/2-j_screw, 0, 0]){
        rotate([90, 0, 0]){
          cylinder(d=M4_hole+1, h=j_w*1.1, center=true);
          translate([0, 0, j_w/2]){
            cylinder(d=M4_nut_e, h=6, center=true, $fn=6);
          }
        }
      }
      // Bend hinge cut
      translate([j_l/2-50, 27.5, 0]){
        scale([1.3, 1, 2]){
          cylinder(d=30, h=j_h*1.1, center=true);
        }
      }
      translate([j_l/2-50, -27.5, 0]){
        scale([1.3, 1, 2]){
          cylinder(d=30, h=j_h*1.1, center=true);
        }
      }
    }

    // Push rod mount
    if(type == 1){// Non threaded rod
      prm_wall = 2;
      prm_l = 20;
      prm_longhole = 10;
      translate([-(j_l+prm_l)/2, 0, 0]){
        difference(){
          cube(size=[prm_l, pr_d+2*prm_wall, j_h], center=true);
          cube(size=[prm_l*1.1, pr_d, j_h*1.1], center=true);
          hull(){
            translate([prm_longhole/2, 0, 0]){
              rotate([90, 0, 0]){
                cylinder(d=M4_hole, h=j_w, center=true);
              }
            }
            translate([-prm_longhole/2, 0, 0]){
              rotate([90, 0, 0]){
                cylinder(d=M4_hole, h=j_w, center=true);
              }
            }
          }
        }
      }
    }
  }

}


module roundedRect(size, radius, edges=[1, 1, 1, 1]){
  x = size[0];
  y = size[1];
  z = size[2];

  linear_extrude(height=z){
    hull(){
        // place 4 circles in the corners, with the given radius
        if(edges[0]){
          translate([(-x/2)+(radius), (-y/2)+(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(-x/2)+(radius/2), (-y/2)+(radius/2), 0])
          square(size=[radius, radius], center=true);
        }
        if(edges[1]){
          translate([(x/2)-(radius), (-y/2)+(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(x/2)-(radius/2), (-y/2)+(radius/2), 0])
          square(size=[radius, radius], center=true);
        }

        if(edges[2]){
          translate([(-x/2)+(radius), (y/2)-(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(-x/2)+(radius/2), (y/2)-(radius/2), 0])
          square(size=[radius, radius], center=true);
        }

        if(edges[3]){
          translate([(x/2)-(radius), (y/2)-(radius), 0])
          circle(r=radius);
        }
        else{
          translate([(x/2)-(radius/2), (y/2)-(radius/2), 0])
          square(size=[radius, radius], center=true);
        }
    }
  }
}